%load IQC description
load SimpleLTIExampleIQC

% basis function for the parameter dependent lyapunov function
% set up for the LTI case
Fbasis = ones(1,1,1); % constant term
Fgrad = zeros(1,0); %gradient of the basis function with respect to the parameters
% bounds on the parameter variation rates
RateBounds = []; %empty means unbounded

FlgPos = 1; % adds P >=0 constraint 
[gam1,info1] = simplenormboundedgain(G,IQClist,Fbasis,Fgrad,RateBounds,FlgPos);

FlgPos = 0; % does not include P >= 0 constraint
[gam2,info2] = simplenormboundedgain(G,IQClist,Fbasis,Fgrad,RateBounds,FlgPos);

disp([gam1, gam2])


%% evaluation
%construct ard factorization of optimal IQC parameterization (info2)

Apsi = IQClist{1}{1}.a;
Bpsi = IQClist{1}{1}.b;
Cpsi1 = IQClist{1}{1}.c;
Dpsi1 = IQClist{1}{1}.d;
Dpsi2 = IQClist{2}{1};

Q = info2.lamopt(1)*Cpsi1'*blkdiag(1,-1)*Cpsi1;
S = info2.lamopt(1)*Cpsi1'*blkdiag(1,-1)*Dpsi1;
R = info2.lamopt(1)*Dpsi1'*blkdiag(1,-1)*Dpsi1 + ...
    info2.lamopt(2)*Dpsi2'*blkdiag(1,-1)*Dpsi2;

[U,T] = schur(R);        
ev = real( diag(T) );
pidx = find( ev>0 );
qidx = find( ev<0 );
W = diag( sqrt([ev(pidx); -ev(qidx)]) );
W = W*U(:,[pidx;qidx])';

p = length(pidx);
q = length(qidx);
Jpq = blkdiag( eye(p), -eye(q) );

X0 = care(Apsi,Bpsi,Q,R,S);
Psi = ss(Apsi,Bpsi,Jpq*inv(W')*(Bpsi'*X0+S'),W);    

%check if modified lyapunov matrix is positive definite
Ptilde = info2.P - blkdiag(X0,0);
eig(Ptilde)